
package Mirror;

# This package contains a function to mirror a directory structure
# at one site to an identical directory structure at another site.
# The code uses a directory object which could rpresent any kind of
# directory structure.  For example, to mirror WWW directories, the
# directory object would be from class URL_Directory 
# (see URL_Directory.pl).
#
# The mirror() function takes two directory objects as arguments,
# $dirlocal and $dirremote.
# $dirlocal and $dirremote are objects with the following methods 
# defined:
#
# FileList(filename)             Returns a list of strings giving file
#                                names.  Just the name, not the path.
# Exists(filename)               Returns True if filename exists in
#                                the directory.
# IsDirectory(filename)          Returns true if filename is itself a
#                                directory.        
# Age(filename)                  Returns the age of filename (days 
#                                since last modification, like -M)
# Time(filename)                 Returns the time at which the age
#                                (above) was computed (in seconds since
#                                1970).
# Copy(local_dir_obj,filename)   Copies filename from remote site to 
#                                local site (called with remote 
#                                directory object, $dirremote->Copy())
# Delete(local_dir_obj,filename) Deletes the local copy of filename
# MakeSubDir(filename)           Create a subdirectory called filename
#                                on the local disk.
# GetSubDir(filename)            Returns a new directory object which
#                                is one directory lower that the object
#                                which calls GetSubDir; the sub 
#                                directory name is filename.
#
#-------------------------------------------------
#
# Example:
#
# For example, to mirror the YPOP directories from MSU to LMSAL, I use
#
# require "URL_Directory.pl";
# require "Mirror.pl";
#
# $dirlocal = new URL_Directory "http://www.space.lockheed.com/cgi-bin/dirlist?/YPOP/","/usr/local/etc/httpd/htdocs/YPOP";
# $dirremote = new URL_Directory "http://www.physics.montana.edu/cgi-bin/dirlist?/YPOP/";
#
# Mirror::Mirror($dirlocal,$dirremote);
#
#-------------------------------------------------
#
# T. Metcalf 1996-December-12

$EXACT = 1;   # If set, do an "exact" mirror so that local files which are not
              # at the remote site, are deleted.

@NOMIRROR = ('.','..');          # File names  to ignore.  This 
                                 # array MUST contain at least '.' 
                                 # and '..', otherwise an infinite
                                 # loop will occur.

grep($MARK{$_}++,@NOMIRROR);   # Process the NOMIRROR array for later use 


sub Mirror {

   my $dirlocal = shift;
   my $dirremote = shift;

   my(@dfiles,$d);

   @dfiles = FileList $dirremote;

   # Remove the @NOMIRROR files from the directory list

   @dfiles = grep(!$MARK{$_},@dfiles);

   # Look for files which need to be mirrored

   foreach $d (@dfiles) {
      # Is it a directory?
      if (IsDirectory $dirremote $d) {
         unless ((Exists $dirlocal $d) && (IsDirectory $dirlocal $d)) {
            # Create the subdirectory in $dirlocal
            if (Exists $dirlocal $d) {
               warn "Remote Directory, Local File name conflict $d\n";
            } else {
               MakeSubDir $dirlocal $d;
            }
         }
         # Recursive search of directories
         my $d1 = GetSubDir $dirlocal $d;
         my $d2 = GetSubDir $dirremote $d;
         &Mirror($d1,$d2); 
      } else {
         # $d is not a directory
         if (Exists $dirlocal $d) {
            # check the age, if the local is older grab the remote copy
            # The remote age is adjusted to the time of the local age
            my $agelocal = Age $dirlocal $d;
            my $ageremote = (Age $dirremote $d) + 
               ((Time $dirlocal $d) - (Time $dirremote $d))/86400.;
            # The constant below adds two seconds
            if ($agelocal > $ageremote+2.3148e-5) {
               # mirror the remote copy
               #my $td = (Time $dirlocal $d) - (Time $dirremote $d);
	       #print "Copy $d\n";
	       Copy $dirremote $dirlocal,$d;
            }
         } else {
            # there is no local copy, so mirror it
	    #print "Copy $d\n";
	    Copy $dirremote $dirlocal,$d;
         }
      }
   }

   # Now check the local files and delete any that do not exist at the
   # remote site.
   if ($EXACT) {
      my @localdfiles = FileList $dirlocal;
      # Remove the @NOMIRROR files from the directory list
      @localdfiles = grep(!$MARK{$_},@localdfiles);
      # Look for files which need to be deleted
      foreach $d (@localdfiles) {
         if (!(Exists $dirremote $d)) {
	    Delete $dirlocal $d;
	 }
      }
   }
}

1

