
package URL_Directory;

# URL_Directory is a class designed to get and hold a directory list
# over the WWW.  When created with the new() method, the URL is 
# accessed.  This will only work with a cgi-bin program which reads
# the directory and spits it out in the correct format.  E.g. 
# http"//www.space.lockheed.com/cgi-bin/dirlist?/YPOP/.
# The second argument to new() is optional and gives the actual 
# path on disk to the WWW directory in the URL.  This is intended to 
# work with the Mirror::mirror (see Mirror.pl).
#
# For example, to mirror the YPOP directories from MSU to LMSAL, I use
#
# require "URL_Directory.pl";
# require "Mirror.pl";
#
# $dirlocal = new URL_Directory "http://www.space.lockheed.com/cgi-bin/dirlist?/YPOP/","/usr/local/etc/httpd/htdocs/YPOP";
# $dirremote = new URL_Directory "http://www.physics.montana.edu/cgi-bin/dirlist?/YPOP/";

# Mirror::Mirror($dirlocal,$dirremote);

#----------------------------------------------
# Methods
#    new(url,path)
#    FileList()              returns list of file names
#    IsDirectory(filename)   true if filename is a directory#
#    IsReadable(filename)    true if filename is readable
#    Exists(filename)        true if filename is in directory
#    Age(filename)           returns age of filename in days since last 
#                               modification
#    Time(filename)          returns the time at which the ages were calculated
#                               (sec since 1970)
#    URL()                   returns URL of directory
#    Update()                re-read the URL directory
#    Update(url)             updates url and reads new directory
#    Path()                  returns path of ht docs
#    Path(path)              sets ht doc path
#    Copy(remote,local,filename)  Copy remote WWW file to local disk
#    Delete(local,filename)  Delete local file
#    MakeSubDir(filename)    make a new subdirectory on disk
#    GetSubDir(filename)     return a new dir object of subdirectory
#    
# Fields:   Don't mess with these directly!
#    URL = string
#    DIR = {name => [mode,age,time]}
#    PATH = string
#
#---------------------------------------
#  T. Metcalf 1996-December-12
#  T. Metcalf 1997-May-5  Added recursive delete of directory contents

require "url_get.pl";

$debug = 0;

sub new {
   my $proto = shift;
   my $url = shift;
   my $path = shift;
   my @files = @_;  # the rest of the arguments are file names to look for
   my $class = ref($proto) || $proto;
   my $self = {};
   if (defined($url)) {
      $self->{URL} = $url;
      $self->{DIR} = &_get_url_dir($url,@files);
      if (defined($path)) {
         $self->{PATH} = $path;
      } else {
         $self->{PATH} = undef;
      }
   } else {
      $self->{URL} = undef;
      $self->{DIR} = {};
      $self->{PATH} = undef;

   }
   bless($self, $class);
   return $self;
}

sub Update {
   my $self = shift;
   my $url = shift;
   if (defined($url)) {
      $self->{URL} = $url;
      $self->{DIR} = &_get_url_dir($url);
   } else {
      if (defined($self->{URL})) {
         $self->{DIR} = &_get_url_dir($self->{URL});
      }
   }
}

sub URL {
   my $self = shift;
   return $self->{URL};
}

sub FileList {
   my $self = shift;
   return keys(%{$self->{DIR}});
}

sub Time {
   my $self = shift;
   my $file = shift;
   return $self->{DIR}{$file}[2];
}

sub IsDirectory {
   my $self = shift;
   my $file = shift;
   my $mode = $self->{DIR}{$file}[0];
   return $mode & 8;
}

sub IsReadable {
   my $self = shift;
   my $file = shift;
   my $mode = $self->{DIR}{$file}[0];
   return $mode & 4;
}

sub Exists {
   my $self = shift;
   my $file = shift;
   return scalar(grep(/^$file$/,keys(%{$self->{DIR}})));
}


sub Age {
   my $self = shift;
   my $file = shift;
   return $self->{DIR}{$file}[1];
}

sub Path {
   my $self = shift;
   my $path = shift;
   if (defined($path)) {
      $self->{PATH} = $path;
   } else {
      $path = $self->{PATH};
   }
   return $path;
}

sub Copy {
   my $remote = shift;
   my $local = shift;
   my $file = shift;

   my $url = $remote->URL;
   $url = $url."/" unless substr($url,length($url)-1,1) EQ "/";
   $url = $url.$file;
   # in case a cgi-bin program is used to give the directory, remove
   # any thing that look like "/cgi-bin/dirlist?" since the path will
   # presumably follow the question mark
   $url =~ s|cgi-bin/\w+\?/||;

   my $path = $local->Path;
   $path = $path."/" unless substr($path,length($path)-1,1) EQ "/";
   my $local_file_name = $path.$file;

   if (!$debug) {
      print "Copy: $url -> $local_file_name\n";
      eval('&url_get($url,$local_file_name)'); # Grab the remote file
      if ($@ ne "") {warn "url_get failed on $local_file_name ($@)\n";}
      if (-e $local_file_name) {
         # The mod time of the local file must be the same as the mod time 
         # of the remote.
         my $modtime = ($remote->Time($file)) - ($remote->Age($file))*86400.;
         utime time,$modtime,$local_file_name; 
      }
   } else {
      print "Copy: $url -> $local_file_name\n";
   }

}

sub Delete {

   my $self = shift;
   my $file = shift;

   my $path = $self->Path;
   $path = $path."/" unless substr($path,length($path)-1,1) EQ "/";
   my $local_file_name = $path.$file;

   if ($debug) {
      print "Delete: $local_file_name\n";
      return 1;
   } else {
      if (-d $local_file_name) {
	 # Is it empty?  If not delete everything recursively
	 my @DList,$DFile;
	 if (opendir DDir, $local_file_name) {
	     @DList = readdir DDir;
             closedir DDir;
             foreach $DFile (@DList) {
		 if ($DFile ne "." && $DFile ne "..") {
                     Delete $self $file."/".$DFile;
                 }
             }
         }
         print "Delete: rmdir $local_file_name\n";
         return rmdir($local_file_name);
      } else {
         print "Delete: unlink $local_file_name\n";
         return unlink($local_file_name);
      }
   }

}

sub MakeSubDir {

   my $self = shift;
   my $file = shift;

   my $path = $self->Path;
   $path = $path."/" unless substr($path,length($path)-1,1) EQ "/";
   my $local_file_name = $path.$file;

   if ($debug) {
      print "MakeSubDir: $local_file_name\n";
      return 1;
   } else {
      return mkdir($local_file_name,0775);
   }

}

sub GetSubDir {

   my $self = shift;
   my $file = shift;

   my $url = $self->URL;
   my $path = $self->Path;
   $url = $url."/" unless substr($url,length($url)-1,1) EQ "/";
   $url = $url.$file;

   $path = $path."/" unless substr($path,length($path)-1,1) EQ "/";
   $path = $path.$file;
   if ($debug) {print "GetSubDir: $url\n";}
   my $d = new URL_Directory $url,$path;

   return $d;

}

sub _get_url_dir {
   my $url = shift;
   my @files = @_;
   my (@dir,@elements,$line);
   my $dir = {};
   my $i;
   if ($debug) {print "Read $url\n";}
   my $url_list=eval('&url_get($url)'); # Grab the remote dir list
   if ($@ ne "") {$url_list=eval('&url_get($url)');}  # try again
   if ($@ ne "") {
      die "url_get failed on $url ($@)\n";
   } else {
      @dir = split(/\n/,$url_list);
   }
   foreach $line (@dir) {
      @elements = split(/\s+/,$line);
      my $filename = $elements[0];
      if (@elements == 4) {
         my $fileOK = 0;
         for ($i=0; $i<scalar(@files); ++$i) {  # Check if filename in list
            if ($filename eq $files[$i]) {$fileOK=1;}
            #if ($debug) {print $filename," ",$files[$i]," ",$fileOK,"\n";}
         }
	 if ($fileOK || scalar(@files) == 0) {
            $dir->{$elements[0]} = [$elements[1],$elements[2],$elements[3]];
	 }
      }
   }
   return $dir;
}

1


